﻿using System.Runtime.InteropServices;

namespace BeeDevelopment.PlayJoy {
	
	/// <summary>
	/// Provides methods for accessing functions in the native C library.
	/// </summary>
	static class Interop {

		/// <summary>Gets the state of the data line.</summary>
		/// <param name="portAddress">Base address of the parallel port.</param>
		/// <returns>The state of the data line.</returns>
		[DllImport("PlayJoy.dll")]
		public static extern bool GetData(short address);

		/// <summary>Gets the state of the acknowledge line.</summary>
		/// <param name="portAddress">Base address of the parallel port.</param>
		/// <returns>The state of the data line.</returns>
		[DllImport("PlayJoy.dll")]
		public static extern bool GetAcknowledge(short address);

		/// <summary>Sets the state of the command line.</summary>
		/// <param name="portAddress">Base address of the parallel port.</param>
		/// <param name="state">The state to set the command line to.</param>
		[DllImport("PlayJoy.dll")]
		public static extern void SetCommand(short portAddress, bool state);

		/// <summary>Sets the state of the select line.</summary>
		/// <param name="portAddress">Base address of the parallel port.</param>
		/// <param name="state">The state to set the select line to.</param>
		[DllImport("PlayJoy.dll")]
		public static extern void SetSelect(short portAddress, bool state);

		/// <summary>Sets the state of the clock line.</summary>
		/// <param name="portAddress">Base address of the parallel port.</param>
		/// <param name="state">The state to set the clock line to.</param>
		[DllImport("PlayJoy.dll")]
		public static extern void SetClock(short portAddress, bool state);

		/// <summary>Begins a data transfer by pulling select low.</summary>
		/// <param name="portAddress">Base address of the parallel port.</param>
		[DllImport("PlayJoy.dll")]
		public static extern void BeginTransfer(short portAddress);

		/// <summary>Ends a data transfer by releasing select high.</summary>
		/// <param name="portAddress">Base address of the parallel port.</param>
		[DllImport("PlayJoy.dll")]
		public static extern void EndTransfer(short portAddress);

		/// <summary>Exchanges a byte between the PlayStation controller and the PC.</summary>
		/// <param name="portAddress">Base address of the parallel port.</param>
		/// <param name="data">The data to exchange.</param>
		/// <returns>True if the transmission was acknowledged, false if it timed out.</returns>
		[DllImport("PlayJoy.dll")]
		public static extern bool ExchangeByte(short portAddress, ref byte data);

		/// <summary>Exchanges a block of data between the PlayStation controller and the PC.</summary>
		/// <param name="portAddress">Base address of the parallel port.</param>
		/// <param name="command">The command byte to send.</param>
		/// <param name="data">The data to exchange (input and output).</param>
		/// <param name="numberOfElements">The size of data in bytes.</param>
		/// <returns>The size of the received packet.</returns>
		[DllImport("PlayJoy.dll")]
		public static extern int SendPacket(short portAddress, ref byte command, byte[] data, int numberOfElements);

	}
}
